﻿/*
    Program ukazujici casove rozdily 2. 
*/

using System;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;

namespace Comparison2
{
    class Program
    {
        private const int numThreads = 4;
        private const int numValues = 50000000;

        private static CountdownEvent cEvent;
        private static int[,] values;
        private static int[] results;

        public static void GenerateValues()
        {
            Random rand = new Random();

            for (int i = 0; i < numThreads; i++)
                for (int j = 0; j < numValues; j++)
                    values[i, j] = rand.Next(10);
        }

        public static void SumTP(object n)
        {
            int row = (int)n;

            int temp = 0;
            for (int i = 0; i < numValues; i++)
                temp += values[row, i];

            results[row] = temp;
            cEvent.Signal();
        }

        public static void SumT(object n)
        {
            int row = (int)n;

            int temp = 0;
            for (int i = 0; i < numValues; i++)
                temp += values[row, i];

            results[row] = temp;
        }

        static void Main(string[] args)
        {
            values = new int[numThreads, numValues];
            results = new int[numThreads];
            Stopwatch sw = new Stopwatch();

            Console.WriteLine("Generování vstupních hodnot");
            GenerateValues();

            Console.WriteLine("Začátek výpočtu");

            sw.Start();
            cEvent = new CountdownEvent(numThreads);
            for (int i = 0; i < numThreads; i++)
            {
                ThreadPool.QueueUserWorkItem(SumTP, i);
            }
            cEvent.Wait();
            sw.Stop();

            Console.WriteLine("Výpočet pomocí třídy ThreadPool: {0,5} ms",
                sw.ElapsedMilliseconds);

            sw.Reset();

            sw.Start();
            Task[] tasks = new Task[numThreads];
            for (int i = 0; i < numThreads; i++)
            {
                tasks[i] = new Task(SumT, i);
                tasks[i].Start();
            }
            Task.WaitAll(tasks);
            sw.Stop();

            Console.WriteLine("Výpočet pomocí třídy Task      : {0,5} ms",
                sw.ElapsedMilliseconds);

            sw.Reset();

            sw.Start();
            Parallel.For(0, numThreads, i => SumT(i));
            sw.Stop();

            Console.WriteLine("Výpočet pomocí třídy Parallel  : {0,5} ms",
                sw.ElapsedMilliseconds);

            sw.Reset();

            sw.Start();
            Thread[] threads = new Thread[numThreads];
            for (int i = 0; i < numThreads; i++)
            {
                threads[i] = new Thread(SumT);
                threads[i].Start(i);
            }
            for (int i = 0; i < numThreads; i++)
                threads[i].Join();
            sw.Stop();

            Console.WriteLine("Výpočet pomocí třídy Thread    : {0,5} ms",
                sw.ElapsedMilliseconds);

            sw.Reset();

            sw.Start();
            for (int i = 0; i < numThreads; i++)
                SumT(i);
            sw.Stop();

            Console.WriteLine("Sekvenční výpočet              : {0,5} ms",
                sw.ElapsedMilliseconds);
        }
    }
}
